' Licensed Materials - Property of IBM
' 5724-T46
' (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
' US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
'On Error Resume Next
'***********************************************************BeginFile************
' Time-stamp: <040212 Louis 13:08 getnewrecs.vbs>
' Name:
'   getnewrecs.vbs
' Purpose:
'   retrieve records appended to a log since the previous run
'   and store them sequentially in a new file
' Notes:
'   getnewrecs.vbs <outfilename> <platformname> <directory> <filename pattern> [<propsname> <headerlineprefix> <datetimeformat>]
'   <outfilename>       Name of temp chunk To be collected
'  <platformname>      unique registry key value To store state of already collected Data
'   <directory>         path of directory To collect
'   <filename pattern>  regex of files To be collected from the directory
'   <propsname>         unique ID of the event source instance
'   <headerlineprefix>  prefix to recognize relevant header lines
'   <datetimeformat>    name of property to put in props file (for IIS)
' History:
'   010515 0.0.1 edwin EE105004: new
'   010528 0.0.10 edwin EE105004: added functionality to read current (open) logfiles as well as old
'                                 already closed files according To a filename pattern.
'   010809 0.0.16 Edwin QE108002: write first record if the logfile is switched
'   030820 <id>  Andrew EE0305013: add dateformat support per sublog, append debug log, 
'                               and store properties in registry dependently on LPI
'   040204 0.0.66 Louis EE0305013: add headerline treatment
'   2004oct27 0.2 VL: WE0407002: VBS actuators do not return exit codes - fixed
'   050224 0.0.74 ihoshko WE0502064: Generic actuator scripts issues
'       050224 AB WE0502064: added checking for arguments number at the very 
'       begining of script 
'***********************************************************EndFile******
Option Explicit      ' Force explicit variable declaration.
On Error Resume Next

' Variable declaration
Dim args             ' The arguments of this program
Dim iCnt             ' Counter for arguments
Dim WshShell         ' The shell to direct commands to
Dim debugLogObject
Dim debugLogFile
Dim InFileName
Dim OutFileName      ' The file containing the collected log data
Dim PltfName         ' a unique name of the platform to create the regkey to store information
Dim Directory        ' Directory which contains the log files
Dim FilenamePattern  ' Filename Pattern of logfiles
Dim PropsFile        ' Name of the *.props file
Dim DateTimeFormatProperty ' Name of property to put into .props file
Dim fso, infile, outfile, record, crmtime, HeaderLinePrefix, CurLine

' Constant declaration
Const regKey = "HKLM\Software\CONSUL Risk Management\CEAActuator\"
Const regprevious = "\previous"           ' key to contain timestamp of last run
Const regfirstrec = "\firstrecord"        ' key to contain first record of last used file
Const reglastline = "\lastline"           ' key to contain last read linenumber of last used file
Const debugLog = "..\log\getnewrecs_"
Const tracing  = True 
Const appendMode = 8

Set args = wscript.Arguments
If args.Count < 4 Then
  If tracing Then
    infoMessage debugLogFile, "Insufficient number of arguments."
  End If
  Call CleanExit(1)
End If

Err.Clear
Set wshShell = wscript.CreateObject("WScript.Shell")

OutFileName = args(0)
PltfName = args(1)
Directory = args(2)
FilenamePattern = args(3)
If args.Count > 4 Then
  PropsFile = args(4)
End If
If args.Count > 5 Then
  DateTimeFormatProperty = args(5)
End If
If args.Count > 6 Then
  HeaderLinePrefix = args(6)
End If

' Initiate logging file
Dim removedProps, append
If tracing Then
  removedProps = Replace(PropsFile, ".props", "")
  append = debugLog & removedProps & ".Log"
  Set debugLogObject = CreateObject("Scripting.FileSystemObject")
  Set debugLogFile   = debugLogObject.OpenTextFile(append, appendMode, True)
  wscript.echo "Opened log file " & append
End If

If tracing Then
  debugLogFile.WriteLine "" ' Log Separator
  For iCnt = 0 To args.Count-1
    infoMessage debugLogFile, "Args " & iCnt & " : " & args(iCnt)
  Next
End If

Set fso = CreateObject("Scripting.FileSystemObject")

' 040205 0.0.66 Louis EE0305013: to be able to perform unittests with
' reproducible results, we must be able to delete the state
' information that previous calls to this script stored in the
' registry.  If the "platform" is equal to "unittest", and if the
' a properties file is specified but does not exist, then the "unittest"
' registry keys are removed before the collecting begins.
If PltfName = "unittest" Then
  If Not fso.FileExists(PropsFile) Then
    ' we delete the registry keys, if they exist
    If tracing Then
      infoMessage debugLogFile, "Unittest mode: removing old registry keys"
    End If
    wshShell.RegDelete regkey & PltfName & "\" & PropsFile & regprevious
    wshShell.RegDelete regKey & PltfName & "\" & PropsFile & reglastline
    wshShell.RegDelete regKey & PltfName & "\" & PropsFile & regfirstrec
  End If
End If

' get timestamp from previous run
Dim previous, lastdate

previous = ReadRegistry(regprevious)
If previous = "" Then
  lastdate=DateAdd("d",-2,Now())
Else
  lastdate=CDate(previous)
End If
If tracing Then
  infoMessage debugLogFile, "Last date from previous run: " & lastdate
End If

' get first record from previous run
Dim firstrecord      ' first record of file, to determine if this is the same file as previous run
firstrecord = ReadRegistry(regfirstrec)
If firstrecord = "" Then
  firstrecord = "first run"
End If

' get last linenumber from previous run
Dim lastline         ' last line read
lastline = ReadRegistry(reglastline)
If lastline = "" Then
  lastline = "1"
End If

' 040212 0.0.68 Louis EE0305013: Note on unittesting:
' This script depends on the CRMTime.dll with the new GetDateTimeFormat
' being registered, but if we register this DLL as part of unittesting,
' then that registration clobbers the one that any prior ISM installation
' on the same system did.  So, we do not register the DLL and instead
' ignore errors related to it (except that we do write a message to
' the log file about it).

' 040212 0.0.68 Louis EE0305013: during unittesting we cannot assume
' that the following object is always present
Err.Clear
Set crmtime = CreateObject("CRMTime.CRMTimeObject")
If Err.Number <> 0 Then
  wshShell.Run "RegSvr32 /s ..\bin\CRMTime.dll", 0, true
  Err.Clear
  Set crmtime = CreateObject("CRMTime.CRMTimeObject")
  If Err.Number <> 0 Then
    If tracing <> 0 Then
      InfoMessage debugLogFile, "no CRMTime.dll found"
    End If
    Call CleanExit(1)
  End If
End If

' get and store date format string to the props file
If PropsFile <> "" Then
  If DateTimeFormatProperty <> "" Then
    Dim str_Format
    ' 040212 0.0.68 Louis EE0305013: during unittesting we cannot assume
    ' that the GetDateTimeFormat method is always present
    crmtime.GetDateTimeFormat str_format
    If Err.Number <> 0 Then
        If tracing <> 0 Then
              InfoMessage debugLogFile, "no GetDateTimeFormat method in CRMTime.dll"
                        End If
                        Call CleanExit(1)
    End If
    UpdateDateFormat DateTimeFormatProperty & "=", str_Format
  Else
    ' 040206 0.0.67 Louis EE0305013: if a property file is specified,
    ' then we must create it if it doesn't yet exist, or else
    ' unittesting fails
    If Not fso.FileExists(PropsFile) Then
      Set outfile = fso.CreateTextFile(PropsFile, True)
      outfile.Close
    End If
  End If
End If

' open outfile
Set outfile = fso.CreateTextFile(OutFileName, True)

' get the filenames to process
Dim folder, fc, matcher, f1count, f1, f1order, f1Idx
Dim CreationTime, LastAccessTime, LastWriteTime
Set folder = fso.GetFolder(Directory)

Set fc = folder.Files

Set matcher = WScript.CreateObject("VBScript.RegExp")


matcher.Pattern = FilenamePattern
matcher.IgnoreCase = True
ReDim f1s(0)
f1count = 0
For Each f1 in fc

  crmtime.GetFileTime f1.Path, CreationTime, LastAccessTime, LastWriteTime  

  if lastdate <= LastWriteTime and matcher.test(f1.Name) Then

    ReDim Preserve f1s(f1count)
    Set f1s(f1count) = f1
    f1count = f1count + 1
  End If
Next

If tracing Then
  InfoMessage debugLogFile, "found " & f1count & " log files to collect"
End If
f1Order = SortLogfiles(f1s)
For f1Idx = 0 To f1count-1
  CollectLogfile f1s(f1Order(f1Idx))
  ' remember the last modified date, unless it is a date in the future
  If f1s(f1Order(f1Idx)).DateLastModified <= Now Then 'Local time/local time comparison OK
    crmtime.GetFileTime f1s(f1Order(f1Idx)).Path, CreationTime, LastAccessTime, LastWriteTime
    previous = LastWriteTime
  End if
Next

outfile.Close

If tracing Then
  InfoMessage debugLogFile, "updating registry:"
  InfoMessage debugLogFile, "previous = " & previous
  InfoMessage debugLogFile, "lastline = " & lastline
  InfoMessage debugLogFile, "firstrecord = " & firstrecord
End If

' store new values in registry
wshShell.RegWrite regkey & PltfName & "\" & PropsFile & regprevious, previous
wshShell.RegWrite regKey & PltfName & "\" & PropsFile & reglastline, lastline, "REG_DWORD"
wshShell.RegWrite regKey & PltfName & "\" & PropsFile & regfirstrec, firstrecord

If tracing Then
  InfoMessage debugLogFile, "done."
End If
Call CleanExit(0)

'=============================================================================================

' do the actual collection of the new records
Sub CollectLogfile(f1)
  ' open infile
  Set infile = fso.OpenTextFile(f1.Path, 1) ' ForReading
  ' read first record from infile  // 2004jul05 AndrewZhdanov: QE0406009: if there is a record
  If Not infile.AtEndOfStream Then
    record = infile.ReadLine
  Else
    record = "no input"
  End If
  ' compare if equal to line in registry (same file)
  ' start at first record if different
  If record & ";" & f1.Path <> firstrecord Then
    lastline = 1
    firstrecord = record &  ";" & f1.Path
    outfile.WriteLine record
  End If
  InfoMessage debugLogFile , (f1.DateLastModified & " " & f1.Path & " line: " & lastline)
  If HeaderLinePrefix <> "" Then ' 040204 0.0.66 Louis EE0305013: header lines
    ' 040204 0.0.66 Louis EE0305013: include the last relevant header line
    Dim HeaderLine
    Do While infile.Line <= lastline And infile.AtEndOfStream <> True
      CurLine = infile.ReadLine
      If Left(CurLine, Len(HeaderLinePrefix)) = HeaderLinePrefix Then ' match
        HeaderLine = CurLine
      End If
    Loop
    If CurLine <> "" Then ' only if we did find a header line in lines >= 2
      outfile.WriteLine HeaderLine
    End If
  Else
    Do While infile.Line <= lastline And infile.AtEndOfStream <> True
      infile.SkipLine
    Loop
  End If
  ' 2005Dec09 AlexanderPanasyuk: WE05MB016: from now do not skipping of log lines 
  ' which begin with unprintable characters
'  Dim NullCount
  ' process record until eof
  Do While infile.AtEndOfStream <> True
    CurLine = infile.ReadLine
    lastline = lastline + 1
'    If Left(CurLine, 1) >= " " Then
      outfile.WriteLine CurLine
'      NullCount = 0
'    Else
      ' 040205 0.0.67 Louis EE0305013: we don't include lines that begin
      ' with unprintable characters
'      NullCount = NullCount + 1
'    End If
  Loop
  ' get last linenumber of infile
  ' 040205 0.0.67 Louis EE0305013: we must correct for the last skipped
  ' unprintable lines
'  lastline = infile.Line - NullCount
  ' 2004jul05 AndrewZhdanov: QE0406009: point right after the last line
'  if NullCount > 0 Then lastline = lastline + 1
  infile.Close
End Sub 

Function Comparef1s(f1, f2)
  Dim LastWriteTime1, LastWriteTime2, dummy1, dummy2
  crmtime.GetFileTime f1.Path, dummy1, dummy2, LastWriteTime1
  crmtime.GetFileTime f2.Path, dummy1, dummy2, LastWriteTime2  
  If LastWriteTime1 > LastWriteTime2 Then
    Comparef1s = 1
  ElseIf LastWriteTime1 < LastWriteTime2 Then
    Comparef1s = -1
  ElseIf f1.Name > f2.Name Then
    Comparef1s = 1
  ElseIf f1.Name < f2.Name Then
    Comparef1s = -1
  Else
    Comparef1s = 0
  End If   
End Function  

'In:  f1s:     file objects
'Out: Long:    sorted order by LastWriteTime and name
Function SortLogfiles(Strings())
  Dim Count, Order1, Order2
  Dim i1, i1max, i2, i2max, i3, SortedLength
  On Error Resume Next
  Count = UBound(Strings) + 1
  On Error GoTo 0
  If Count > 0 Then
    ReDim Order1(Count - 1)
    ReDim Order2(Count - 1)
    For i1 = 0 To Count - 1      'Start order
      Order1(i1) = i1
    Next
    SortedLength = 1
    While SortedLength < Count
      i1 = 0
      i2 = i1 + SortedLength
      i3 = 0
      While i1 < Count
        i1max = i2
        i2max = i2 + SortedLength
        If i2max > Count Then
          i2max = Count
        End If   
        While i1 < i1max And i2 < i2max
          Select Case Comparef1s(f1s(Order1(i1)), f1s(Order1(i2)))
          Case 1:
            Order2(i3) = Order1(i2)
            i2 = i2 + 1
          Case Else
            Order2(i3) = Order1(i1)
            i1 = i1 + 1
          End Select
          i3 = i3 + 1
        Wend
        For i1 = i1 To i1max - 1
          Order2(i3) = Order1(i1)
          i3 = i3 + 1
        Next
        For i2 = i2 To i2max - 1
          Order2(i3) = Order1(i2)
          i3 = i3 + 1
        Next
        i1 = i1 + SortedLength
        i2 = i2 + SortedLength
        If i2 > Count Then i2 = Count
      Wend
      Order1 = Order2
      SortedLength = SortedLength * 2
    Wend
    SortLogfiles = Order1
  End If
End Function

' update or insert the date format to the *.props file
'In:  date_header:  header of line at the file, for ex. date_header=MM/DD/YY
'In:  str_format:   format of date, for ex. datetimeformat=str_format
Sub UpdateDateFormat(date_header, str_format)

  Dim fso, fprops, appendMode, writeMode, readMode
  Dim lineNum, bFind, str_line
  Dim propsArr()

  ' define values
  readMode = 1
  writeMode = 2
  appendMode = 8

  Set fso = CreateObject("Scripting.FileSystemObject")

  Set fprops = fso.OpenTextFile(PropsFile, readMode, True)
  lineNum = 0
  bFind = false
  ' find line with header date if present
  Do While fprops.AtEndOfStream <> True
    str_line = fprops.ReadLine
    if 1 = InStr(1,str_line,date_header,1) then
      bFind = true
    else
      lineNum = lineNum + 1
      Redim Preserve propsArr(lineNum)
      propsArr(lineNum) = str_line
    end if
  Loop
  fprops.Close()

  ' write date format string
  if bFind then
    Dim i
    Set fprops = fso.OpenTextFile(PropsFile, writeMode, True)
    for i = 1 to lineNum 
     fprops.WriteLine propsArr(i)
   Next
    fprops.WriteLine date_header & str_format
  else
    Set fprops = fso.OpenTextFile(PropsFile, appendMode, True)
    fprops.WriteLine date_header & str_format
  end if
  fprops.Close()
  Erase propsArr
End Sub

' infoMessage()
'
Sub infoMessage(outputFile, text)
  wscript.echo "Message: " & text
  outputFile.WriteLine Now & " " & text
End Sub

' ReadRegistry
' EE0305013: read LPI record if possible, or generic one otheriwise
' note: must On Error GoTo 0 after the function
Function ReadRegistry(key)
  On Error Resume Next ' no On Error GoTo 0 here

  ReadRegistry = wshShell.RegRead(regKey & PltfName & "\" & PropsFile & key)
  If Err.Number <> 0 Then  ' read genereic value
    Err.Clear

    infoMessage debugLogFile, "Reading old style property '" & key & "' from registry"
    ReadRegistry = wshShell.RegRead(regKey & PltfName & key)
  End If
End Function

'
' CleanExit(error_code)
'
Sub CleanExit(error_code)
  On Error Resume Next
  If tracing Then
    debugLogFile.Close()
  End If
        WScript.Quit error_code
End Sub

